@@ -1,5 +1,6 @@
+import contextlib
+import time
 import threading
-from queue import Queue, Empty
 
 from pyjoystick.stash import Stash
 from pyjoystick.utils import PeriodicThread
@@ -10,12 +11,6 @@ class ThreadEventManager(object):
                  button_repeater=None, activity_timeout=0.01):
         super().__init__()
 
-        if add_joystick is not None:
-            self.add_joystick = add_joystick
-        if remove_joystick is not None:
-            self.remove_joystick = remove_joystick
-        if handle_key_event is not None:
-            self.handle_key_event = handle_key_event
         if alive is None:
             alive = threading.Event()
 
@@ -30,8 +25,14 @@ class ThreadEventManager(object):
         self.event_lock = threading.RLock()
         self.event_list = Stash()
 
-        if self.button_repeater is not None:
-            self.button_repeater = button_repeater
+        if add_joystick is not None:
+            self.add_joystick = add_joystick
+        if remove_joystick is not None:
+            self.remove_joystick = remove_joystick
+        if handle_key_event is not None:
+            self.handle_key_event = handle_key_event
+        if button_repeater is not None:
+            self.set_button_repeater(button_repeater)
 
     def add_joystick(self, joy):
         """Save the added joystick."""
@@ -45,20 +46,20 @@ class ThreadEventManager(object):
         """Function to handle key event happens"""
         pass
 
-    @property
-    def button_repeater(self):
+    def get_button_repeater(self):
         """Return the button repeater."""
         return self._button_repeater
 
-    @button_repeater.setter
-    def button_repeater(self, value):
+    def set_button_repeater(self, value):
         """Set the button repeater."""
         self._button_repeater = value
         try:
-            self.button_repeater.key_repeated = self._update_key_event
+            self._button_repeater.key_repeated = self._update_key_event
         except:
             pass
 
+    button_repeater = property(get_button_repeater, set_button_repeater)
+
     def save_joystick(self, joy):
         """Save the added joystick."""
         self.joysticks.append(joy)
@@ -118,6 +119,77 @@ class ThreadEventManager(object):
                 # Run the callback handler
                 self.handle_key_event(k)
 
+    @contextlib.contextmanager
+    def run_during(self):
+        """Context manager to temporarily run the manager if the manager is not already running."""
+        if not self.is_running():
+            self.start()
+            yield
+            self.stop()
+        else:
+            yield
+
+    def wait(self, conditional=None, timeout=float('inf'), sleep_func=None):
+        """Wait for the given timeout or conditional function to return false.
+
+        Args:
+            conditional (callable)[None]: Callable that returns True to keep waiting.
+            timeout (float/int)[float('inf')]: Time for when to stop waiting.
+            sleep_func (callable)[None]: How to sleep and wait (ex. time.sleep(0.001)).
+        """
+        if sleep_func is None:
+            sleep_func = lambda: time.sleep(0.01)
+        if conditional is None:
+            conditional = lambda: True
+        elif not callable(conditional):
+            conditional = lambda: conditional
+
+        start = time.time()
+        while (time.time() - start) < timeout and conditional():
+            sleep_func()
+
+    def find_key(self, joysticks=None, key_types=None, timeout=float("inf"), sleep_func=None):
+        """Wait and return the next key that is pressed.
+
+        Args:
+            joysticks (list/Joystick)[None]: Joystick(s) to allow events for.
+            key_types (list/KeyTypes)[None]: List of key type. Found with Key.KeyTypes.Axis
+            timeout (float/int)[float('inf')]: Timeout to wait.
+            sleep_func (callable)[None]: How to sleep and wait (time.sleep(0.01)).
+
+        Returns:
+            key (Key)[None]: If found return the first Key else return None.
+        """
+        if joysticks is None:
+            joysticks = []
+        elif not isinstance(joysticks, list):
+            joysticks = list(joysticks)
+        if key_types is None:
+            key_types = []
+
+        data = {'found key': None}
+
+        def filter_find_key(key):
+            is_joystick = len(joysticks) == 0 or key.joystick in joysticks
+            is_key_type = len(key_types) == 0 or key.has_keytype(key.keytype, key_types)
+            is_valid_value = key.keytype != key.AXIS or abs(key.value) > 0
+            if is_joystick and is_key_type and is_valid_value and data['found key'] is None:
+                data['found key'] = key
+
+        def is_not_found():
+            return data['found key'] is None
+
+        # Change the event handler
+        old_handle_key_event, self.handle_key_event = self.handle_key_event, filter_find_key
+
+        with self.run_during():
+            self.wait(is_not_found, timeout, sleep_func)
+
+        # Reset event handler
+        self.handle_key_event = old_handle_key_event
+
+        return data['found key']
+
     def run(self, event_loop, add_joystick, remove_joystick, handle_key_event, alive=None, button_repeater=None):
         """Run the an event loop to process SDL Events.
 
@@ -157,6 +229,7 @@ class ThreadEventManager(object):
         self.worker.alive = self.alive  # stop when this event stops
         self.worker.daemon = True
         self.worker.start()
+        return self
 
     def stop(self):
         """Stop running the event loop."""
@@ -174,6 +247,7 @@ class ThreadEventManager(object):
         except:
             pass
         self.worker = None
+        return self
 
     def __enter__(self):
         if not self.is_running():
